from SimulatorFramework import SimUtils,TileType,Tile,TileAssembly
from SimulatorFramework import Frontier
from PyTASTileType import *
import os
import sys
import random


#===============================================================================
# N  = 0
# NE = 1
# E  = 2
# SE = 3
# S  = 4
# SW = 5
# W  = 6
# NW = 7
# ALLDIRS = 8
# DIRSTRS = ("N", "NE", "E", "SE", "S", "SW", "W", "NW")
#===============================================================================
#NBRS = [(0,1),(1,1),(1,0),(1,-1),(0,-1),(-1,-1),(-1,0),(-1,1)]
NBRS = [(0,1),(1,0),(0,-1),(-1,0),]

#----------------------------------------------------------------------------
        
class PyTASAssembly(TileAssembly.Assembly):

    def __init__(self, tileWidths=100, tileHeights=100):
        TileAssembly.Assembly.__init__(self, tileWidths, tileHeights)

        self.growthIncrement = 500
        self.temperature = 0
        self.FrontierHashTable = dict()
        self.bHas3D = False

    def ClearSpace(self):
        TileAssembly.Assembly.ClearSpace(self)
        self.FrontierHashTable = dict()

    def GetFittingTilesForLoc(self, coords):
        retSet = None
        count = 0
        nbrGlues = []
        for n in NBRS:
            nbrCoords = (coords[0]+n[0], coords[1]+n[1])
            nbrTile = self.GetLoc(nbrCoords, True)
            if not nbrTile is None:
                oppDir = (count + 2) % 4
                nbrGlues.append(nbrTile.type.sides[oppDir].Hash())
                #if retSet is None:
                #    retSet = nbrTile.type.matches[oppDir]
                #else:
                #    retSet = retSet.intersection(nbrTile.type.matches[oppDir])
            else:
                nbrGlues.append(None)
            count = count + 1

        signature = str(nbrGlues)
        if signature in self.FrontierHashTable:
            return self.FrontierHashTable[signature]

        for t in self.tileTypes:
            totStr = 0
            for n in range(len(NBRS)):
                if not nbrGlues[n] is None:
                    if nbrGlues[n] == self.tileTypes[t].sides[n].Hash():
                        totStr = totStr + self.tileTypes[t].sides[n].strength
            if totStr >= self.temperature:
                if retSet is None:
                    retSet = set()
                retSet.add(t)
        self.FrontierHashTable[signature] = retSet
        return retSet
        
    def LoadTileSystem(self, fileName, backgroundColor=None):
        if len(fileName) < 1:
            return False
        elif os.path.isabs(fileName) == False:
            fileName = "./" + fileName
            fileName = os.path.abspath(fileName)

        #try:
        os.stat(fileName)
        rFile = open(fileName, "r")
        SimUtils.globalApp.OutputMessage("Using tile system definition file '" + fileName + "'")
        lines = rFile.readlines()
        rFile.close()
        
        self.tileTypes = dict()
        self.seedPatch = []
        self.frontier = set()
        self.temperature = 2    # Default temperature = 2
        self.bHas3D = False
        
        tdsFileName = lines[0].strip()
        if os.path.isabs(tdsFileName) == False:
            tdsFileName = os.path.dirname(fileName) + "/" + tdsFileName
            tdsFileName = os.path.abspath(tdsFileName)
        self.LoadTileSet(tdsFileName, backgroundColor=backgroundColor)

        for line in lines[1:]:
            if line.startswith("Temperature="):
                temperatureStr = line[len("Temperature="):]
                self.temperature = int(temperatureStr)
            else:
                seedLocArr = line.split(" ")
                seedTileName = seedLocArr[0]
                seedTileXLoc = int(seedLocArr[1])
                seedTileYLoc = int(seedLocArr[2])
                seedTileZLoc = 0
                if len(seedLocArr) > 3:
                    seedTileZLoc = int(seedLocArr[3])
                    if not seedTileZLoc == 0:
                        self.bHas3D = True
                self.seedPatch.append((seedTileName, (seedTileXLoc, seedTileYLoc, seedTileZLoc)))

        # If seed had 3D coords, make sure all tile types are 3D
        if self.bHas3D == True:
            for ttName in self.tileTypes:
                self.tileTypes[ttName].SetDimensions(3)

        # Load tile set, then verify that tile types referenced in the seed actually exist
        midPt = self.PlaceSeed()

        self.tileSystemDef = dict()
        self.tileSystemDef['tdsFile'] = tdsFileName
        self.tileSystemDef['temperature'] = self.temperature
        return midPt
        
    def CreateTileTypeImages(self, backgroundColor=None):
        for ttypeName in self.tileTypes:
            self.tileTypes[ttypeName].MakeImage(backgroundColor=backgroundColor)
        
    def LoadTileSet(self, tdsFileName, backgroundColor=None):
        SimUtils.globalApp.OutputMessage("Reading tile set from '" + tdsFileName + "'")
        os.stat(tdsFileName)
        rFile = open(tdsFileName, "r")
        lines = rFile.readlines()
        rFile.close()

        i = 0
        ttype = PyTASTileType()
        while i < len(lines):
            lineArr = lines[i].strip().split(" ")
            if lineArr[0] == "TILENAME":
                ttype.name = lineArr[1]
            elif lineArr[0] == "LABEL" and len(lineArr) > 1:
                ttype.label = lineArr[1]
            elif lineArr[0] == "TILECOLOR" and len(lineArr) > 1:
                ttype.color = lineArr[1]
            elif lineArr[0] == "NORTHLABEL" and len(lineArr) > 1:
                ttype.sides[N].label = lineArr[1]
            elif lineArr[0] == "EASTLABEL" and len(lineArr) > 1:
                ttype.sides[E].label = lineArr[1]
            elif lineArr[0] == "SOUTHLABEL" and len(lineArr) > 1:
                ttype.sides[S].label = lineArr[1]
            elif lineArr[0] == "WESTLABEL" and len(lineArr) > 1:
                ttype.sides[W].label = lineArr[1]
            elif lineArr[0] == "UPLABEL" and len(lineArr) > 1:
                self.bHas3D = True
                ttype.SetDimensions(3)
                ttype.sides[U].label = lineArr[1]
            elif lineArr[0] == "DOWNLABEL" and len(lineArr) > 1:
                self.bHas3D = True
                ttype.SetDimensions(3)
                ttype.sides[D].label = lineArr[1]
            elif lineArr[0] == "NORTHBIND" and len(lineArr) > 1:
                ttype.sides[N].strength = int(lineArr[1])
            elif lineArr[0] == "EASTBIND" and len(lineArr) > 1:
                ttype.sides[E].strength = int(lineArr[1])
            elif lineArr[0] == "SOUTHBIND" and len(lineArr) > 1:
                ttype.sides[S].strength = int(lineArr[1])
            elif lineArr[0] == "WESTBIND" and len(lineArr) > 1:
                ttype.sides[W].strength = int(lineArr[1])
            elif lineArr[0] == "UPBIND" and len(lineArr) > 1:
                self.bHas3D = True
                ttype.SetDimensions(3)
                ttype.sides[U].strength = int(lineArr[1])
            elif lineArr[0] == "DOWNBIND" and len(lineArr) > 1:
                self.bHas3D = True
                ttype.SetDimensions(3)
                ttype.sides[D].strength = int(lineArr[1])
            elif lineArr[0] == "CREATE":
                ttype.ParseColor()
                if self.tileTypes.get(ttype.name) == None:
                    self.tileTypes[ttype.name] = ttype
                elif self.tileTypes.get(ttype.name) != None:
                    SimUtils.globalApp.OutputMessage("Warning, Duplicate tile name: " + ttype.name + "\tThis may cause unexpected behavior")
                    duplicateNum = 1
                    while self.tileTypes.get(ttype.name + "(" + str(duplicateNum) + ")") != None:
                        duplicateNum += 1
                    ttype.name = ttype.name + "(" + str(duplicateNum) + ")"
                    self.tileTypes[ttype.name] = ttype
                ttype = PyTASTileType()
            i = i + 1

        if self.bHas3D == True:
            for ttName in self.tileTypes:
                self.tileTypes[ttName].SetDimensions(3)

        # MJP: This greatly slows the loading of large tile sets, but increases
        #      the initial speed of assembly until all tile type images have been
        #      created and cached
        #self.CreateTileTypeImages(backgroundColor=backgroundColor)

        SimUtils.globalApp.OutputMessage("Loaded tile set with size: " + str(len(self.tileTypes)))

        return True

