import wx
import os
import random
import ast
import sys
import thread
import threading
import wx.aui
from wx.lib.pubsub import pub
from PyTASAssembly import *
from PyTASRubberband import *
from SimulatorFramework import Frontier
from SimulatorFramework import SimCanvas


# IDs
PYTAS_ID_STOP          = 100
PYTAS_ID_FOR           = 101
PYTAS_ID_FF            = 102
PYTAS_ID_ZOOMIN        = 103
PYTAS_ID_ZOOMOUT       = 104
PYTAS_ID_ZOOM_MODE     = 105
PYTAS_ID_SELECT_MODE   = 106
PYTAS_ID_DELETE        = 107
PYTAS_ID_UNDO          = 108

PYTAS_SM_STEP = 0
PYTAS_SM_FULL = 1


#----------------------------------------------------------------------------

class PyTASCanvas(SimCanvas.Canvas):
    def __init__(self, parent, idIn, frame, size, app):
        self.drawUpdates = SimCanvas.SIMFW_FULLDISPLAY
        SimCanvas.Canvas.__init__(self, parent, idIn, frame, size, app)

        self.backgroundColor = wx.Colour((255, 255, 128, 255))
        self.backgroundBrush = wx.Brush(self.backgroundColor)
        self.backgroundPen = wx.Pen(self.backgroundColor)
        self.drawBitmaps = True
        self.stepMode = PYTAS_SM_STEP
        self.numTilesPerStep = 1

    def CreateNewAssembly(self):
        self.sideLen = SimCanvas.CELLSIZE
        self.assembly = PyTASAssembly()
        self.UpdateSizeVars()
        #self.RedrawAssembly()
        self.CanvasSizeUpdate((0,0))

    def DoSave(self):
        #saveFileDialog = wx.FileDialog(self.window, "Save As", "", "", 
        #                               "Tile system files (*.txt)|*.txt",
        #                               wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        #saveFileDialog.ShowModal()
        #fileName = saveFileDialog.GetPath()
        #saveFileDialog.Destroy()

        #self.saveSnapshot()

        dlg = wx.MessageDialog(self, "Sorry, this hasn't been implemented yet..")
        dlg.ShowModal()
        dlg.Destroy()
        return
    
        currFile = "output.txt"
        newFile = wx.FileSelector("Save assembly to file...", currFile, currFile, ".txt", "*.txt", wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT)
        if len(newFile) > 0:
            fOut = open(newFile, "w")
            iCount = 0
            jCount = 0
            for i in range(self.numHoriz):
                for j in range(self.numVert):
                    if self.grid.IsEmpty(i, j) == False:
                        pt = self.grid.GetCenterOffsetOfType(i, j)
                        if pt.x == 0 and pt.y == 0:
                            name = self.grid.GetType(i, j)
                            outStr = name + "," + str(i) + "," + str(j) + "\n"
                            fOut.write(outStr)
            fOut.close()

    def saveSnapshot(self, dcSource):
        # based largely on code posted to wxpython-users by Andrea Gavana 2006-11-08
        #size = dcSource.Size
    
        # Create a Bitmap that will later on hold the screenshot image
        # Note that the Bitmap must have a size big enough to hold the screenshot
        # -1 means using the current default colour depth
#         bmp = wx.EmptyBitmap(self.width, self.height)
#     
#         # Create a memory DC that will be used for actually taking the screenshot
#         memDC = wx.MemoryDC()
#     
#         # Tell the memory DC to use our Bitmap
#         # all drawing action on the memory DC will go to the Bitmap now
#         memDC.SelectObject(bmp)
#     
#         # Blit (in this case copy) the actual screen on the memory DC
#         # and thus the Bitmap
#         memDC.Blit( 0, # Copy to this X coordinate
#             0, # Copy to this Y coordinate
#             self.width, # Copy this width
#             self.height, # Copy this height
#             dcSource, # From where do we copy?
#             0, # What's the X offset in the original DC?
#             0  # What's the Y offset in the original DC?
#             )
        
        bmp = wx.EmptyBitmap(self.width, self.height)
        #wdc = wx.WindowDC(self)
        memDC = wx.MemoryDC(bmp)

        bg = wx.Brush("WHITE")
        memDC.SetBackground(bg)
        memDC.Clear()
        #memDC.Blit(0, 0, self.width, self.height, wdc, 0, 0)
        
        for i in range(self.assembly.minX, self.assembly.maxX+1):
            for j in range(self.assembly.minY, self.assembly.maxY+1):
                loc = self.assembly.GetLoc((i,j))
                if loc is not None:
                    if self.NameMatchesPatterns(loc.type.name) == True:
                        self.DrawAtCoords(memDC, i, j, self.assembly.GetLoc((i,j)))
        
        # Select the Bitmap out of the memory DC by selecting a new
        # uninitialized Bitmap
        memDC.SelectObject(wx.NullBitmap)
        imageFileName = 'D:/src/Wang/SubstitutionTilings/fall 2016/snapshots/saved.bmp'
        bmp.SaveFile(imageFileName, wx.BITMAP_TYPE_BMP)
        
        ##pdfFileName = 'D:/src/Wang/SubstitutionTilings/fall 2016/snapshots/saved.pdf'
        #=======================================================================
        # c = canvas.Canvas(pdfFileName, pagesize=letter)
        # width, height = letter
        # c.drawImage(imageFileName, inch, height - 2 * inch) # Who needs consistency?
        # c.showPage()
        # c.save()
        #=======================================================================
        #=======================================================================
        # doc = SimpleDocTemplate(pdfFileName, pagesize=(self.width,self.height))
        # parts = []
        # parts.append(Image(imageFileName))
        # doc.build(parts)
        #=======================================================================
    
        #img = bmp.ConvertToImage()
        #img.SaveFile('D:/src/Wang/SubstitutionTilings/fall 2016/snapshots/saved.png', wx.BITMAP_TYPE_PNG)

    def ShowHelp(self):
        info = wx.AboutDialogInfo()
        info.SetName("PyTAS aTAM Simulator")
        info.SetVersion("0.1.0")
        helpStr = "\nList of commands:\n\n"
        helpStr = helpStr + "  1 : select tile system definition file to load\n"
        helpStr = helpStr + "  s : save current assembly\n"
        helpStr = helpStr + "  r : reset current tile system to seed\n"
        helpStr = helpStr + "  f : simulate one step\n"
        helpStr = helpStr + "  g : begin fast forward simulation\n"
        helpStr = helpStr + "  q : quit fast forward simulation\n"
        helpStr = helpStr + "  p : print list of frontier locations\n"
        helpStr = helpStr + "  t : toggle display of grid lines\n"
        helpStr = helpStr + "  d : toggle display updates of tiles to assembly\n"
        helpStr = helpStr + " +/-: zoom in/out\n"
        helpStr = helpStr + "<left mouse click>:  zoom in on clicked location\n"
        helpStr = helpStr + "<right mouse click>:  zoom out of clicked location\n"
        helpStr = helpStr + "  h : show this help dialog"
        info.SetDescription(helpStr)
        wx.AboutBox(info)
